# Generated by roxygen2: do not edit by hand

S3method(oe_get_keys,character)
S3method(oe_get_keys,default)
S3method(oe_get_keys,sf)
S3method(oe_match,bbox)
S3method(oe_match,character)
S3method(oe_match,default)
S3method(oe_match,numeric)
S3method(oe_match,sf)
S3method(oe_match,sfc)
S3method(oe_match_pattern,bbox)
S3method(oe_match_pattern,character)
S3method(oe_match_pattern,numeric)
S3method(oe_match_pattern,sf)
S3method(oe_match_pattern,sfc)
S3method(print,oe_key_values_list)
export(get_default_osmconf_ini)
export(oe_clean)
export(oe_download)
export(oe_download_directory)
export(oe_find)
export(oe_get)
export(oe_get_boundary)
export(oe_get_keys)
export(oe_get_network)
export(oe_match)
export(oe_match_pattern)
export(oe_providers)
export(oe_read)
export(oe_update)
export(oe_vectortranslate)
export(read_poly)
